/************************************************************************
 *                                                                      *
 *  gactive.c                                                            *
 *  -draw the map 'activity' to graphics (in SGI-machines)              *
 *                                                                      *
 *  Version 1.0                                                         *
 *  Date: 30 Jun 1992                                                    *
 *                                                                      *
 *  NOTE: This program package is copyrighted in the sense that it      *
 *  may be used for scientific purposes. The package as a whole, or     *
 *  parts thereof, cannot be included or used in any commercial         *
 *  application without written permission granted by its producents.   *
 *  No programs contained in this package may be copied for commercial  *
 *  distribution.                                                       *
 *                                                                      *
 *  All comments  concerning this program package may be sent to the    *
 *  e-mail address 'lvq@cochlea.hut.fi'.                                *
 *                                                                      *
 ************************************************************************/

#include <stdio.h>
#include <float.h>
#include <math.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "lvq_pak.h"

/*
 * This is the routine which calculates the neuron activity 
 */

void calc_active(struct entries *data, struct entries *codes, float *act)
{
  int index, i;
  float diff, dist, activity;
  struct entries *codetmp;
  struct entries *datatmp;

  codetmp = codes;
  index = 0;
  while (codetmp != NULL) {
    activity = 0;
    datatmp = data;
    while (datatmp != NULL) {
      dist = 0;
      for(i = 0; i < codes->dimension; i++) {
        diff = (codetmp->points[i] - datatmp->points[i]);
	dist += diff * diff;
      }
      activity += exp(-4.0 * dist);
      datatmp = datatmp->next;
    }
  act[index] = activity;
  codetmp = codetmp->next;
  index++;
  }
}

/*
 * I have made some minor changes to draw smaller circles
 */

void draw_active(struct entries *data, struct entries *codes, float *act, char *winname)
{
  short val;
  int cv;
  int i;
  int xsize, ysize;
  int xp, yp;
  int height = 20;
  int index;
  int offset = 0;
  int oplane = 0;
  long gwin;
  float maxval, minval;
  float diff, diffsf, difference;
  struct entries *codetmp;
  struct entries *datatmp;
  Device dev;
  short reds[64], greens[64], blues[64];

  if (codes->topol = topol_type("hexa")) {
    offset = 10;
    height = 18;
  }

  xsize = 20 * codes->xdim + offset;
  ysize = height * codes->ydim;

  foreground();
  prefsize(xsize, ysize);
  /* Open the graphics window */
  gwin = winopen(winname);

  /* Change the coordinate (0,0) to upper left corner */
  translate(0, ysize, 0);
  scale(1.0, -1.0, 0.0);

  /* Get mouse for quit detection */
  qdevice(LEFTMOUSE);

  for (i = 0; i < 64; i++) {
    cv = 255 * (float) i / 64.0;
    getmcolor((long) (i + 64), &reds[i], &greens[i], &blues[i]);
    mapcolor((long) (i + 64), cv, cv, cv);
  }

  dev = LEFTMOUSE;
  do {

    if (dev != REDRAW)

    /* Find the lower and upper limits for gray level scaling */
    minval = FLT_MAX;
    maxval = FLT_MIN;
    codetmp = codes;
    index = 0;
    while (codetmp != NULL) {
      if (maxval < act[index])
	maxval = act[index];
      if (minval > act[index])
	minval = act[index];
      codetmp = codetmp->next;
      index++;
    }
  
    color(WHITE);
    clear();
  
    /* Show the gray level circles */
    index = 0;
    codetmp = codes;
    while (codetmp != NULL) {
      xp = 20 * (index % codes->xdim) + 10;
      yp = height * (index / codes->xdim) + 10;
      if (((index / codes->xdim) % 2)) xp += offset;
  
      cv = (int) (52 * (act[index] - minval) /
			(maxval - minval));
      color(cv+70);
      circfi(xp, yp, 10);
  
      codetmp = codetmp->next;
      index++;
    }

  } while ((dev = qread(&val)) != LEFTMOUSE);

  for (i = 0; i < 64; i++) {
    mapcolor((long) (i + 64), reds[i], greens[i], blues[i]);
  }

  gexit();
}

main(int argc, char **argv)
{
  int plane;
  char *in_code_file;
  char *in_data_file = NULL;
  char winname[100];
  float act[10000];
  struct entries *codes;
  struct entries *data;

  in_code_file = extract_parameter(argc, argv, IN_CODE_FILE, ALWAYS);
  in_data_file = extract_parameter(argc, argv, IN_DATA_FILE, ALWAYS);
  verbose((int) oatoi(extract_parameter(argc, argv, VERBOSE, OPTION), 1));

  label_not_needed(1);
  if (verbose(-1) > 1)
    fprintf(stdout, "Codebook entries are read from file %s\n", in_code_file);
  codes = read_entries(in_code_file);

  if (codes->topol < topol_type("hexa")) {
    printf("File %s is not a map file\n", in_code_file);
    exit(0);
  }

  if (verbose(-1) > 1)
    fprintf(stdout, "Data entries are read from file %s\n", in_data_file);
  data = read_entries(in_data_file);

  if (data->dimension > codes->dimension) {
    errormsg("Dimensions in data and codebook files are different");
    exit(0);  
  }

  sprintf(winname, "%s - %s", in_code_file, in_data_file);

  calc_active(data, codes, act);
  draw_active(data, codes, act, winname);

  return(0);
}
